/************************************ <lalVerbatim file="PrintFrequencySeriesCV">
Author: Whelan, J. T.
$Id: PrintFrequencySeries.m4,v 1.3 2001/02/10 22:41:39 jolien Exp $
************************************* </lalVerbatim> */

/* <lalLaTeX>

\subsection{Module \texttt{PrintFrequencySeries.c}}
\label{ss:PrintFrequencySeries.c}

Print a $\langle\mbox{datatype}\rangle$FrequencySeries object into a
file.  For use in non-production and test code only.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{PrintFrequencySeriesCP}
\index{\texttt{LALZPrintFrequencySeries()}}
\index{\texttt{LALCPrintFrequencySeries()}}
\index{\texttt{LALDPrintFrequencySeries()}}
\index{\texttt{LALSPrintFrequencySeries()}}
\index{\texttt{LALI2PrintFrequencySeries()}}
\index{\texttt{LALI4PrintFrequencySeries()}}
\index{\texttt{LALI8PrintFrequencySeries()}}
\index{\texttt{LALU2PrintFrequencySeries()}}
\index{\texttt{LALU4PrintFrequencySeries()}}
\index{\texttt{LALU8PrintFrequencySeries()}}
\index{\texttt{LALPrintFrequencySeries()}}

\subsubsection*{Description}

Each member of this family of functions prints the elements of
$\langle\mbox{datatype}\rangle$\verb+FrequencySeries+ into a file.
Note: the file name is specified using a character string.  This
function is for debugging use only: its arguments do not conform to
LAL standards so it should not be used in any real analysis codes.

\subsubsection*{Algorithm}

\subsubsection*{Uses}

\begin{verbatim}
LALCHARCreateVector()
LALCHARDestroyVector()
LALUnitAsString()
\end{verbatim}

\subsubsection*{Notes}

This function's arguments do not conform to the LAL spec.  For this
reason it should only be used for debugging purposes in test
functions, not in any production code.

Additionally, since printf cannot handle INT8 as integers, the
functions \verb&LALI8PrintFrequencySeries()& and
\verb&LALU8PrintFrequencySeries()& use a typecast to REAL8 and are
thus only valid for numbers between around $-10^{15}$ and $10^{15}$.

The first four lines of the file are a header containing:
\begin{enumerate}
\item the name of the series
\item heterodyning information, if any
\item the starting epoch, relative to the GPS reference epoch (1980 January 6)
\item the units expressed in terms of the basic SI units
\item column labels
\end{enumerate}
after which come the data, one per line.

The output  is two or three tab-separated columns: the first
column is the frequency in hertz corresponding to the row in question;
for real and integer frequency series, the second column is the value
of the series; for complex frequency series, the second column is the
real part and the third the imaginary part of the value.

Note that the frequency given is the physical frequency.  In the case
of a heterodyned frequency series, this is the heterodyning frequency
plus the frequency offset.  A frequency series of length $[N]$ is
assumed to be packed so that the 0th element corresponds to zero
frequency offset, elements 1 through $[N/2]$ to positive frequency
offsets (in ascending order), and elements $N-[N/2]$ to $N-1$ to
negative frequency offsets (also in ascending order, so that the
frequency corresponding to the $N-1$st element is just below that of
the 0th element).  If $N$ is even, the element in position $N/2$ is
assumed to correspond both the maximum poitive and negative frequency
offset.

\vfill{\footnotesize\input{PrintFrequencySeriesCV}}

</lalLaTeX> */


#include <lal/LALStdlib.h>
#include <stdio.h>
#include <lal/LALDatatypes.h>
#include <lal/PrintFTSeries.h>

void LALCHARCreateVector( LALStatus *, CHARVector **, UINT4 );
void LALCHARDestroyVector( LALStatus *, CHARVector ** );
void LALUnitAsString( LALStatus *status, CHARVector *output,
                      const LALUnit *input );

enum { LALUnitTextSize = sizeof("10^-32768 m^-32768/32767 kg^-32768/32767 "
				"s^-32768/32767 A^-32768/32767 " 
				"K^-32768/32767 strain^-32768/32767 "
				"count^-32768/32767") };

/* <lalVerbatim file="PrintFrequencySeriesNRCSID"> */
NRCSID( PRINTFREQUENCYSERIESC, "$Id: PrintFrequencySeries.m4,v 1.3 2001/02/10 22:41:39 jolien Exp $" );
/* </lalVerbatim> */
























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALZPrintFrequencySeries ( COMPLEX16FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  COMPLEX16 *data;
  FILE *fp;
  COMPLEX16 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a COMPLEX16Sequence */
  /* series->data->data is a pointer to COMPLEX16 */

  /* Make a COMPLEX16 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tRe(Value)\tIm(Value)\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\t%g\n",f,data->re,data->im);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a COMPLEX16 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\t%g\n",f,data->re,data->im);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\t%g\n",f,data->re,data->im);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALCPrintFrequencySeries ( COMPLEX8FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  COMPLEX8 *data;
  FILE *fp;
  COMPLEX8 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a COMPLEX8Sequence */
  /* series->data->data is a pointer to COMPLEX8 */

  /* Make a COMPLEX8 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tRe(Value)\tIm(Value)\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\t%g\n",f,data->re,data->im);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a COMPLEX8 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\t%g\n",f,data->re,data->im);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\t%g\n",f,data->re,data->im);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALDPrintFrequencySeries ( REAL8FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  REAL8 *data;
  FILE *fp;
  REAL8 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a REAL8Sequence */
  /* series->data->data is a pointer to REAL8 */

  /* Make a REAL8 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\n",f,*data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a REAL8 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\n",f,*data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\n",f,*data);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALSPrintFrequencySeries ( REAL4FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  REAL4 *data;
  FILE *fp;
  REAL4 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a REAL4Sequence */
  /* series->data->data is a pointer to REAL4 */

  /* Make a REAL4 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\n",f,*data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a REAL4 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\n",f,*data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%g\n",f,*data);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALI2PrintFrequencySeries ( INT2FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  INT2 *data;
  FILE *fp;
  INT2 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a INT2Sequence */
  /* series->data->data is a pointer to INT2 */

  /* Make a INT2 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a INT2 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALI4PrintFrequencySeries ( INT4FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  INT4 *data;
  FILE *fp;
  INT4 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a INT4Sequence */
  /* series->data->data is a pointer to INT4 */

  /* Make a INT4 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a INT4 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALI8PrintFrequencySeries ( INT8FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  INT8 *data;
  FILE *fp;
  INT8 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a INT8Sequence */
  /* series->data->data is a pointer to INT8 */

  /* Make a INT8 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%0.0f\n",f,(REAL8) *data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a INT8 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%0.0f\n",f,(REAL8) *data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%0.0f\n",f,(REAL8) *data);
    }
  }

  fclose(fp);

  return;
}

/* Note that LALI8PrintFrequencySeries does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */
























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALU2PrintFrequencySeries ( UINT2FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  UINT2 *data;
  FILE *fp;
  UINT2 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a UINT2Sequence */
  /* series->data->data is a pointer to UINT2 */

  /* Make a UINT2 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a UINT2 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALU4PrintFrequencySeries ( UINT4FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  UINT4 *data;
  FILE *fp;
  UINT4 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a UINT4Sequence */
  /* series->data->data is a pointer to UINT4 */

  /* Make a UINT4 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a UINT4 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g %i\n",f,*data);
    }
  }

  fclose(fp);

  return;
}

























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALU8PrintFrequencySeries ( UINT8FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  UINT8 *data;
  FILE *fp;
  UINT8 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a UINT8Sequence */
  /* series->data->data is a pointer to UINT8 */

  /* Make a UINT8 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%0.0f\n",f,(REAL8) *data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a UINT8 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%0.0f\n",f,(REAL8) *data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%0.0f\n",f,(REAL8) *data);
    }
  }

  fclose(fp);

  return;
}

/* Note that LALU8PrintFrequencySeries does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */
























;








/* <lalVerbatim file="PrintFrequencySeriesCP"> */
void LALPrintFrequencySeries ( REAL4FrequencySeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 f;
  REAL4 *data;
  FILE *fp;
  REAL4 *endOfSeq, *lastInBand;
  INT4 halfLength;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a REAL4Sequence */
  /* series->data->data is a pointer to REAL4 */

  /* Make a REAL4 pointer which points to the first memory address not
   * belonging to the sequence
   */
  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
  fprintf(fp,"%s\n",series->name);
  if (series->epoch.gpsSeconds && series->epoch.gpsNanoSeconds) {
    fprintf(fp,"Epoch is %d seconds, %d nanoseconds\n",
            series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  }
  else {
    fprintf(fp,"\n");
  }
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"Units are (%s)\n",unitString->data);
  fprintf(fp,"Freq (Hz)\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  if (series->f0 == 0) {	
    for ( f = 0, data = series->data->data;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%f\n",f,*data);
    }	
  }
  else {
    halfLength = series->data->length/2;   /* rounded down */
  /* Make a REAL4 pointer which points to the memory address of
   * the highest frequency represented in the series
   */
    lastInBand = series->data->data + halfLength;
    for ( f = series->f0, data = series->data->data;
          data <= lastInBand;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%f\n",f,*data);
    }	
    for ( f = series->f0 - series->deltaF * halfLength,
                data = endOfSeq - halfLength;
          data < endOfSeq;
          f += series->deltaF, ++data )
    {
      fprintf(fp,"%g\t%f\n",f,*data);
    }
  }

  fclose(fp);

  return;
}

